@@ -24,33 +24,39 @@ typedef enum {
 
 typedef struct ORBit_RootObject_struct *ORBit_RootObject;
 
-typedef void (* ORBit_RootObject_DestroyFunc)(ORBit_RootObject obj);
+typedef void (* ORBit_RootObject_DestroyFunc) (ORBit_RootObject obj);
 
 typedef struct _ORBit_RootObject_Interface {
-  ORBit_RootObject_Type type;
-  ORBit_RootObject_DestroyFunc destroy;
+	ORBit_RootObject_Type        type;
+	ORBit_RootObject_DestroyFunc destroy;
 } ORBit_RootObject_Interface;
 
 struct ORBit_RootObject_struct {
-  const ORBit_RootObject_Interface * interface;
+	const ORBit_RootObject_Interface *interface;
   int refs;
 };
 
 #define ORBIT_REFCOUNT_STATIC -10
 
-/* Used to determine whether the refcount is valid or not */
-#define ORBIT_REFCOUNT_MAX (1<<20)
-
-#define ORBIT_ROOT_OBJECT(obj) ((ORBit_RootObject)(obj))
-#define ORBIT_ROOT_OBJECT_TYPE(obj) (((ORBit_RootObject)(obj))->interface->type)
-
 void     ORBit_RootObject_init      (ORBit_RootObject obj,
 				     const ORBit_RootObject_Interface *interface);
 
-gpointer ORBit_RootObject_duplicate (gpointer obj);
-void     ORBit_RootObject_release   (gpointer obj);
-void     ORBit_RootObject_release_T (gpointer obj);
+#ifdef ORBIT2_INTERNAL_API
+
+/* Used to determine whether the refcount is valid or not */
+#define ORBIT_REFCOUNT_MAX (1<<20)
+
+#define ORBIT_ROOT_OBJECT(obj)      ((ORBit_RootObject)(obj))
+#define ORBIT_ROOT_OBJECT_TYPE(obj) (((ORBit_RootObject)(obj))->interface->type)
+
+
+gpointer ORBit_RootObject_duplicate   (gpointer obj);
+gpointer ORBit_RootObject_duplicate_T (gpointer obj);
+void     ORBit_RootObject_release     (gpointer obj);
+void     ORBit_RootObject_release_T   (gpointer obj);
 
 extern GMutex *ORBit_RootObject_lifecycle_lock;
 
+#endif /* ORBIT2_INTERNAL_API */
+
 #endif
