@@ -1,10 +1,9 @@
-#include "config.h"
-
+#include <config.h>
 #include <string.h>
-
+#include <sys/uio.h>
 #include "giop-private.h"
 #include <orbit/GIOP/giop.h>
-#include <sys/uio.h>
+#include "../util/orbit-purify.h"
 
 #define GIOP_CHUNK_ALIGN 8
 #define GIOP_CHUNK_SIZE (GIOP_CHUNK_ALIGN * 256)
@@ -321,13 +320,10 @@ giop_send_buffer_align (GIOPSendBuffer *buf, gulong boundary)
 	/* 2. Do the alignment */
 	if (align_amt) {
 
-		if (buf->indirect_left < align_amt) {
-			align_amt -= buf->indirect_left;
+		if (buf->indirect_left < align_amt)
 			get_next_indirect (buf, 0);
-		}
-#ifdef ORBIT_PURIFY
-		memset (buf->indirect, 0, align_amt);
-#endif
+
+		p_memzero (buf->indirect, align_amt);
 		giop_send_buffer_append_real (buf, buf->indirect, align_amt);
 
 		buf->indirect      += align_amt;
@@ -363,19 +359,15 @@ giop_send_buffer_append_aligned (GIOPSendBuffer *buf,
 	   more aggressively here */
 	giop_send_buffer_align (buf, align_len);
   
-	if (buf->indirect_left < align_len) {
-		g_assert (buf->indirect_left == 0);
+	if (buf->indirect_left < align_len)
 		get_next_indirect (buf, 0);
-	}
 
 	indirect = buf->indirect;
 
 	if (mem)
 		memcpy (indirect, mem, align_len);
-#ifdef ORBIT_PURIFY
 	else
-		memset (indirect, 0, align_len);
-#endif
+		p_memzero (indirect, align_len);
 
 	giop_send_buffer_append_real (buf, indirect, align_len);
 	
@@ -385,6 +377,15 @@ giop_send_buffer_append_aligned (GIOPSendBuffer *buf,
 	return indirect;
 }
 
+/**
+ * giop_send_buffer_write:
+ * @buf: the buffer to write
+ * @cnx: the connection to write it to.
+ * 
+ * Writes @buf to @cnx as a block.
+ * 
+ * Return value: 0 on sucess, non 0 on error.
+ **/
 int
 giop_send_buffer_write (GIOPSendBuffer *buf, GIOPConnection *cnx)
 {
@@ -448,3 +449,32 @@ giop_send_buffer_use (GIOPVersion giop_version)
 
 	return retval;
 }
+
+void
+giop_send_buffer_append_string (GIOPSendBuffer *buf,
+				const char     *str)
+{
+	CORBA_unsigned_long len;
+
+	len = strlen (str) + 1;
+
+	/* FIXME: inline me ? */
+	giop_send_buffer_align (buf, 4);
+
+	/* be cleverer for short strings */
+	if (buf->indirect_left >= 4 + len) {
+		guchar *indirect = buf->indirect;
+
+		memcpy (indirect, &len, 4);
+		memcpy (indirect + 4, str, len);
+
+		giop_send_buffer_append_real (buf, indirect, 4 + len);
+	
+		buf->indirect      += 4 + len;
+		buf->indirect_left -= 4 + len;
+	} else {
+		giop_send_buffer_append_copy (buf, &len, 4);
+		giop_send_buffer_append (buf, str, len);
+	}
+}
+
