@@ -1,3 +1,4 @@
+#include <config.h>
 #include <ctype.h>
 #include <string.h>
 #include <popt.h>
@@ -7,18 +8,82 @@
 #include "../orbit-init.h"
 #include "../poa/orbit-poa-export.h"
 #include "../util/orbit-options.h"
+#include "../util/orbit-purify.h"
+#include "iop-profiles.h"
+#include "orb-core-private.h"
 #include "orbhttp.h"
 
 extern ORBit_option orbit_supported_options[];
+
+void
+ORBit_ORB_start_servers (CORBA_ORB orb)
+{
+	LINCProtocolInfo *info;
+
+	for (info = linc_protocol_all (); info->name; info++) {
+		GIOPServer           *server;
+
+		if (!ORBit_proto_use (info->name))
+			continue;
+
+		server = giop_server_new (orb->default_giop_version,
+					  info->name, NULL, NULL, 0, orb);
+		if (server) {
+			orb->servers = g_slist_prepend (orb->servers, server);
+
+			if (!(info->flags & LINC_PROTOCOL_SECURE)) {
+				if (!ORBit_proto_use ("SSL"))
+					continue;
+
+				server = giop_server_new (
+					orb->default_giop_version, info->name,
+					NULL, NULL, LINC_CONNECTION_SSL, orb);
+
+				if (server)
+					orb->servers = g_slist_prepend (orb->servers, server);
+			}
+#ifdef DEBUG
+			fprintf (stderr, "ORB created giop server '%s'\n", info->name);
+#endif
+		}
+#ifdef DEBUG
+		else
+			fprintf (stderr, "ORB failed to create giop server '%s'\n", info->name);
+#endif
+	}
+
+	orb->profiles = IOP_start_profiles (orb);
+}
+
+static void
+ORBit_ORB_shutdown_servers (CORBA_ORB orb)
+{
+	IOP_shutdown_profiles (orb->profiles);
+	orb->profiles = NULL;
+
+	g_slist_foreach (orb->servers, (GFunc) g_object_unref, NULL);
+	g_slist_free (orb->servers); 
+	orb->servers = NULL;
+}
+
+static void
+ORBit_service_list_free_ref (gpointer         key,
+			     ORBit_RootObject objref,
+			     gpointer         dummy)
+{
+	ORBit_RootObject_release (objref);
+}
+
 static void
 CORBA_ORB_release_fn (ORBit_RootObject robj)
 {
 	CORBA_ORB orb = (CORBA_ORB)robj;
 
 	g_ptr_array_free (orb->adaptors, TRUE);
+
 	g_hash_table_destroy (orb->initial_refs);
 
-	g_free (orb);
+	p_free (orb, struct CORBA_ORB_type);
 }
 
 GMutex *ORBit_RootObject_lifecycle_lock = NULL;
@@ -29,20 +94,24 @@ ORBit_locks_initialize (void)
 	ORBit_RootObject_lifecycle_lock = linc_mutex_new ();
 }
 
+static CORBA_ORB _ORBit_orb = NULL;
+
 CORBA_ORB
 CORBA_ORB_init (int *argc, char **argv,
 		CORBA_ORBid orb_identifier,
 		CORBA_Environment *ev)
 {
-	static CORBA_ORB retval = NULL;
+	CORBA_ORB retval;
 	static ORBit_RootObject_Interface orb_if = {
 		ORBIT_ROT_ORB,
 		CORBA_ORB_release_fn
 	};
 
-	if (retval)
-		return (CORBA_ORB) CORBA_Object_duplicate (
-			(CORBA_Object) retval, ev);
+	if ((retval = _ORBit_orb))
+		return ORBit_RootObject_duplicate (retval);
+
+	/* the allocation code uses the bottom bit of this pointer */
+	g_assert (ORBIT_ALIGNOF_CORBA_POINTER > 2);
 
 	ORBit_option_parse (argc, argv, orbit_supported_options);
 
@@ -54,7 +123,7 @@ CORBA_ORB_init (int *argc, char **argv,
 
 	ORBit_RootObject_init (&retval->root_object, &orb_if);
 	/* released by CORBA_ORB_destroy */
-	ORBit_RootObject_duplicate (retval);
+	_ORBit_orb = ORBit_RootObject_duplicate (retval);
 
 	ORBit_genrand_init (&retval->genrand);
 	retval->default_giop_version = GIOP_LATEST;
@@ -62,8 +131,7 @@ CORBA_ORB_init (int *argc, char **argv,
 	retval->adaptors = g_ptr_array_new ();
 	ORBit_init_internals (retval, ev);
 
-	return (CORBA_ORB) CORBA_Object_duplicate (
-		(CORBA_Object) retval, ev);
+	return ORBit_RootObject_duplicate (retval);
 }
 
 CORBA_char *
@@ -178,24 +246,18 @@ CORBA_ORB_string_to_object(CORBA_ORB _obj,
 }
 
 void
-CORBA_ORB_create_list(CORBA_ORB _obj, const CORBA_long count,
-		      CORBA_NVList * new_list,
-		      CORBA_Environment * ev)
+CORBA_ORB_create_list (CORBA_ORB          obj,
+		       const CORBA_long   count,
+		       CORBA_NVList      *new_list,
+		       CORBA_Environment *ev)
 {
-  CORBA_NVList new;
+	CORBA_NVList new;
 
-  new = g_new0 (struct CORBA_NVList_type, 1);
-  if (new==NULL)
-    goto new_alloc_failed;
+	new = g_new0 (struct CORBA_NVList_type, 1);
 
-  new->list = g_array_new(FALSE, TRUE, sizeof(CORBA_NamedValue));
+	new->list = g_array_new (FALSE, TRUE, sizeof (CORBA_NamedValue));
 
-  *new_list = new;
-	
-  return;
-
- new_alloc_failed:
-  CORBA_exception_set_system(ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
+	*new_list = new;
 }
 
 void
@@ -244,59 +306,79 @@ CORBA_ORB_get_service_information(CORBA_ORB _obj,
   return CORBA_FALSE;
 }
 
-static void
-servlist_add_id(gpointer key, gpointer value, gpointer data)
-{
-  CORBA_ORB_ObjectIdList *retval = data;
+struct ORBit_service_list_info {
+	CORBA_ORB_ObjectIdList *list;
+	CORBA_long              index;
+};
 
-  retval->_buffer[retval->_maximum++] = CORBA_string_dup(key);
+static void
+ORBit_service_list_add_id (CORBA_string                    key,
+			   gpointer                        value,
+			   struct ORBit_service_list_info *info)
+{
+	info->list->_buffer [info->index++] = CORBA_string_dup (key);
 }
 
 CORBA_ORB_ObjectIdList *
-CORBA_ORB_list_initial_services(CORBA_ORB _obj,
-				CORBA_Environment *ev)
+CORBA_ORB_list_initial_services (CORBA_ORB          orb,
+				 CORBA_Environment *ev)
 {
-  CORBA_ORB_ObjectIdList *retval;
-  retval = CORBA_ORB_ObjectIdList__alloc();
-  if(_obj->initial_refs)
-    {
-      retval->_length = g_hash_table_size(_obj->initial_refs);
-      retval->_maximum = 0;
-      retval->_buffer =
-	CORBA_sequence_CORBA_ORB_ObjectId_allocbuf(retval->_length);
-      g_hash_table_foreach(_obj->initial_refs, servlist_add_id, retval);
-      retval->_release = CORBA_TRUE;
-    }
-  else
-    {
-      retval->_length = 0;
-      retval->_buffer = NULL;
-    }
-  return retval;
+	CORBA_ORB_ObjectIdList         *retval;
+
+	retval = CORBA_ORB_ObjectIdList__alloc();
+
+	if (orb->initial_refs) {
+		struct ORBit_service_list_info *info;
+
+		info = g_alloca (sizeof (struct ORBit_service_list_info));
+
+		info->index = 0;
+		info->list  = retval;
+
+		retval->_length  = g_hash_table_size (orb->initial_refs);
+		retval->_maximum = retval->_length;
+		retval->_buffer  = CORBA_sequence_CORBA_ORB_ObjectId_allocbuf (
+					retval->_length);
+
+		g_hash_table_foreach (orb->initial_refs,
+				      (GHFunc)ORBit_service_list_add_id,
+				      info);
+
+		retval->_release = CORBA_TRUE;
+
+		g_assert (info->index = retval->_length);
+	}
+	else {
+		retval->_length = 0;
+		retval->_buffer = NULL;
+	}
+
+	return retval;
 }
 
 CORBA_Object
-CORBA_ORB_resolve_initial_references (CORBA_ORB          _obj,
+CORBA_ORB_resolve_initial_references (CORBA_ORB          orb,
 				      const CORBA_char  *identifier,
 				      CORBA_Environment *ev)
 {
-  ORBit_InitialReference *val;
-  if(!_obj->initial_refs)
-    return CORBA_OBJECT_NIL;
-  val = g_hash_table_lookup(_obj->initial_refs, identifier);
+	CORBA_Object objref;
 
-  if(val)
-    return CORBA_Object_duplicate(val->objref, ev);
+	if (!orb->initial_refs)
+		return CORBA_OBJECT_NIL;
 
-  return CORBA_OBJECT_NIL;
+	objref = g_hash_table_lookup (orb->initial_refs, identifier);
+
+	return ORBit_RootObject_duplicate (objref);
 }
 
 static CORBA_TypeCode
-CORBA_TypeCode_allocate(void)
+CORBA_TypeCode_allocate (void)
 {
-  CORBA_TypeCode tc = g_new0(struct CORBA_TypeCode_struct,1);
-  ORBit_RootObject_init(&tc->parent, &ORBit_TypeCode_epv);
-  return ORBit_RootObject_duplicate(tc);
+	CORBA_TypeCode tc = g_new0 (struct CORBA_TypeCode_struct, 1);
+
+	ORBit_RootObject_init (&tc->parent, &ORBit_TypeCode_epv);
+
+	return ORBit_RootObject_duplicate (tc);
 }
 
 CORBA_TypeCode
@@ -791,17 +873,17 @@ CORBA_ORB_shutdown (CORBA_ORB            orb,
 		    const CORBA_boolean  wait_for_completion,
 		    CORBA_Environment   *ev)
 {
-	PortableServer_POA root_poa = g_ptr_array_index (orb->adaptors, 0);
+	PortableServer_POA root_poa;
 
+	root_poa = g_ptr_array_index (orb->adaptors, 0);
 	if (root_poa) {
-		PortableServer_POA_destroy (root_poa, TRUE, wait_for_completion, ev);
+		PortableServer_POA_destroy (
+			root_poa, TRUE, wait_for_completion, ev);
 		if (ev->_major)
 			return;
 	}
 
-	g_slist_foreach (orb->servers, (GFunc)g_object_unref, NULL);
-	g_slist_free (orb->servers); 
-	orb->servers = NULL;
+	ORBit_ORB_shutdown_servers (orb);
 
 	giop_connection_remove_by_orb (orb);
 
@@ -814,6 +896,9 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 {
 	PortableServer_POA root_poa;
 
+	g_assert (_ORBit_orb == orb);
+	_ORBit_orb = NULL;
+
 	if (orb->life_flags & ORBit_LifeF_Destroyed)
 		return;
 
@@ -822,18 +907,59 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 		return;
 
 	root_poa = g_ptr_array_index (orb->adaptors, 0);
-	if (root_poa) {
-		if (((ORBit_RootObject)root_poa)->refs != 1)
-			g_warning ("CORBA_ORB_destroy: Application still has %d refs to RootPOA.",
-				   ((ORBit_RootObject)root_poa)->refs - 1);
+	if (root_poa &&
+	    ((ORBit_RootObject) root_poa)->refs != 1) {
+		g_warning ("CORBA_ORB_destroy: Application still has %d "
+			   "refs to RootPOA.",
+			   ((ORBit_RootObject)root_poa)->refs - 1);
+		CORBA_exception_set_system (
+			ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
+	}
 
-		ORBit_RootObject_release (root_poa);
-		g_ptr_array_index (orb->adaptors, 0) = NULL;
+	g_hash_table_foreach (orb->initial_refs,
+			      (GHFunc)ORBit_service_list_free_ref,
+			      NULL);
+
+	ORBit_RootObject_release (orb->default_ctx);
+	orb->default_ctx = CORBA_OBJECT_NIL;
+
+	{
+		int i;
+		int leaked_poas = 0;
+
+		/* Each poa has a ref on the ORB */
+		for (i = 0; i < orb->adaptors->len; i++) {
+			PortableServer_POA poa;
+
+			poa = g_ptr_array_index (orb->adaptors, i);
+
+			if (poa)
+				leaked_poas++;
+		}
+
+		if (leaked_poas) {
+			g_warning ("CORBA_ORB_destroy: leaked '%d' POAs", leaked_poas);
+			CORBA_exception_set_system (
+				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
+		}
+
+		if (((ORBit_RootObject)orb)->refs != 2 + leaked_poas) {
+			g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
+				   ((ORBit_RootObject)orb)->refs - 1 - leaked_poas);
+			CORBA_exception_set_system (
+				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
+		}
 	}
 
 	orb->life_flags |= ORBit_LifeF_Destroyed;
 
 	ORBit_RootObject_release (orb);
+
+	/* At this stage there should be 1 ref left in the system -
+	 * on the ORB */
+	if (ORBit_RootObject_shutdown ())
+		CORBA_exception_set_system (
+			ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 }
 
 CORBA_Policy
@@ -869,36 +995,27 @@ CORBA_ORB_lookup_value_factory (CORBA_ORB          _obj,
   return CORBA_OBJECT_NIL;
 }
 
-/* ORBit extension */
 void
-CORBA_ORB_set_initial_reference (CORBA_ORB orb, CORBA_ORB_ObjectId identifier,
-				 ORBit_InitialReference *val, CORBA_Environment *ev)
+ORBit_set_initial_reference (CORBA_ORB  orb,
+			     gchar     *identifier,
+			     gpointer   objref)
 {
-	ORBit_InitialReference *findval;
-	char *findkey = NULL;
-
 	if (!orb->initial_refs)
 		orb->initial_refs = g_hash_table_new (g_str_hash, g_str_equal);
 
-	if (g_hash_table_lookup_extended (
-		orb->initial_refs, identifier,
-		(gpointer *)&findkey, (gpointer *)&findval) &&
-	    findval->free_name) {
+	if (g_hash_table_lookup (orb->initial_refs, identifier))
 		g_hash_table_remove (orb->initial_refs, identifier);
-		g_free (findkey);
-	}
 
-	g_hash_table_insert (
-		orb->initial_refs,
-		val->free_name?g_strdup(identifier):identifier,
-		val);
+	g_hash_table_insert (orb->initial_refs,
+			     identifier,
+			     ORBit_RootObject_duplicate (objref));
 }
 
 void
-ORBit_ORB_forw_bind(CORBA_ORB orb, CORBA_sequence_CORBA_octet *okey,
-		    CORBA_Object oref, CORBA_Environment *ev)
+ORBit_ORB_forw_bind (CORBA_ORB orb, CORBA_sequence_CORBA_octet *okey,
+		     CORBA_Object oref, CORBA_Environment *ev)
 {
-  g_warning("ORBit_ORB_forw_bind NYI");
+	g_warning("ORBit_ORB_forw_bind NYI");
 }
 
 /*
@@ -938,3 +1055,4 @@ static ORBit_option orbit_supported_options[] = {
 	{"ORBIIOPSSL",      ORBIT_OPTION_BOOLEAN, &orbit_use_ssl},
 	{NULL,              0,                    NULL},
 };
+
