@@ -1,8 +1,12 @@
+#include <config.h>
+#include <string.h>
+#include <stdlib.h>
 #include <orbit/orbit.h>
 #include <orb-core-export.h>
 #include <orbit-debug.h>
-#include <string.h>
-#include <stdlib.h>
+#include "../util/orbit-purify.h"
+#include "../orb-core/orbit-debug.h"
+#include "poa-macros.h"
 
 #include "orbit-poa.h"
 
@@ -61,8 +65,7 @@ ORBit_POAList_add_child (gpointer key, gpointer value, gpointer data)
 	PortableServer_POA      poa    = value;
 	PortableServer_POAList *retval = data;
 
-	retval->_buffer[retval->_maximum++] =
-		CORBA_Object_duplicate ((CORBA_Object)poa, NULL);
+	retval->_buffer[retval->_maximum++] = ORBit_RootObject_duplicate (poa);
 }
 
 /* PortableServer_Current interface */
@@ -74,7 +77,7 @@ ORBit_POACurrent_free_fn (ORBit_RootObject obj_in)
 	ORBit_RootObject_release_T (poacur->orb);
 	poacur->orb = NULL;
 
-	g_free (poacur);
+	p_free (poacur, struct PortableServer_Current_type);
 }
 
 static const ORBit_RootObject_Interface ORBit_POACurrent_epv = {
@@ -114,23 +117,7 @@ ORBit_POACurrent_get_object (PortableServer_Current  obj,
 		return NULL;
 	}
 
-	return (ORBit_POAObject)obj->orb->current_invocations->data;
-}
-
-CORBA_unsigned_long
-ORBit_classinfo_lookup_id (const char *type_id)
-{
-	PortableServer_ClassInfo *ci;
-
-	if (!ORBit_class_assignments)
-		return 0;
-
-	ci = g_hash_table_lookup (ORBit_class_assignments, type_id);
-
-	if (!ci)
-		return 0;
-
-	return *ci->class_id;
+	return (ORBit_POAObject) obj->orb->current_invocations->data;
 }
 
 PortableServer_ClassInfo *
@@ -205,16 +192,16 @@ remove_cb (PortableServer_ObjectId *oid,
 	   gpointer                 dummy)
 {
 	if (pobj->life_flags & ORBit_LifeF_Destroyed) {
-		g_free (pobj);
+		p_free (pobj, struct ORBit_POAObject_type);
 		return TRUE;
 	}
 	return FALSE;
 }
 
 gboolean
-ORBit_POA_is_inuse(PortableServer_POA poa,
-		   CORBA_boolean consider_children,
-		   CORBA_Environment *ev)
+ORBit_POA_is_inuse (PortableServer_POA poa,
+		    CORBA_boolean      consider_children,
+		    CORBA_Environment *ev)
 {
 	if (poa->use_cnt > 0) 
 		return CORBA_TRUE;
@@ -307,9 +294,7 @@ ORBit_POA_oid_to_obj (PortableServer_POA             poa,
 {
 	ORBit_POAObject pobj;
 
-	/*
-	 * FIXME, FIXME, FIXME
-	 */
+	/* FIXME, FIXME, FIXME */
 	if (poa->p_servant_retention != PortableServer_RETAIN) {
 		if (ev)
 			CORBA_exception_set (
@@ -336,8 +321,10 @@ ORBit_POA_set_life (PortableServer_POA poa,
 		    CORBA_boolean      etherealize_objects,
 		    int                action_do)
 {
-	if ((poa->life_flags 
-	     & (ORBit_LifeF_DeactivateDo|ORBit_LifeF_DestroyDo)) == 0) {
+	if ((poa->life_flags &
+	     (ORBit_LifeF_DeactivateDo |
+	      ORBit_LifeF_DestroyDo)) == 0) {
+
 		if (etherealize_objects)
 			poa->life_flags |= ORBit_LifeF_DoEtherealize;
 	}
@@ -426,7 +413,7 @@ ORBit_POA_destroy (PortableServer_POA poa,
 	/* each objref holds a POAObj, and each POAObj holds a ref 
 	 * to the POA. In addition, the app can hold open refs
 	 * to the POA itself. */
-	numobjs = poa->oid_to_obj_map ? g_hash_table_size(poa->oid_to_obj_map) : 0;
+	numobjs = poa->oid_to_obj_map ? g_hash_table_size (poa->oid_to_obj_map) : 0;
 	g_assert (((ORBit_RootObject)poa)->refs > numobjs);
 
 	poa->life_flags |= ORBit_LifeF_Destroyed;
@@ -444,40 +431,44 @@ ORBit_POA_destroy (PortableServer_POA poa,
     has no affect of children POAs.
 **/
 CORBA_boolean
-ORBit_POA_deactivate(PortableServer_POA poa, CORBA_boolean etherealize_objects,
-		     CORBA_Environment *ev)
+ORBit_POA_deactivate (PortableServer_POA poa,
+		      CORBA_boolean      etherealize_objects,
+		      CORBA_Environment *ev)
 {
-    CORBA_boolean	done = CORBA_TRUE;
+	CORBA_boolean done = CORBA_TRUE;
 
-    ORBit_POA_set_life(poa, etherealize_objects, ORBit_LifeF_DeactivateDo);
-    if ( poa->life_flags & ORBit_LifeF_Deactivated )
-	return TRUE;	/* already did it */
-    if ( poa->life_flags & ORBit_LifeF_Deactivating )
-	return FALSE;	/* recursion */
-    poa->life_flags |= ORBit_LifeF_Deactivating;
+	ORBit_POA_set_life (poa, etherealize_objects, ORBit_LifeF_DeactivateDo);
+	if (poa->life_flags & ORBit_LifeF_Deactivated)
+		return TRUE;	/* already did it */
+	if (poa->life_flags & ORBit_LifeF_Deactivating)
+		return FALSE;	/* recursion */
+	poa->life_flags |= ORBit_LifeF_Deactivating;
 
-    /* bounce all pending requested (OBJECT_NOT_EXIST
-     * exceptions raised); none should get requeued. */
-    ORBit_POA_handle_held_requests(poa);
-    g_assert( poa->held_requests == 0 );
+	/* bounce all pending requested (OBJECT_NOT_EXIST
+	 * exceptions raised); none should get requeued. */
+	ORBit_POA_handle_held_requests (poa);
+	g_assert (poa->held_requests == 0);
 
-    if ( poa->p_servant_retention == PortableServer_RETAIN ) {
-	TraverseInfo	info;
-	info.num_in_use = 0;
-	info.do_deact = 1;
-	info.poa = poa;
-	info.do_etherealize = (poa->life_flags&ORBit_LifeF_DoEtherealize)
-		  		?CORBA_TRUE:CORBA_FALSE;
-	info.is_cleanup = TRUE;
-    	g_assert( poa->oid_to_obj_map );
-	g_hash_table_foreach(poa->oid_to_obj_map, (GHFunc)traverse_cb, &info);
-	g_hash_table_foreach_remove(poa->oid_to_obj_map, (GHRFunc)remove_cb, NULL);
-    	done = info.num_in_use == 0 ? CORBA_TRUE : CORBA_FALSE;
-    }
-    if ( done )
-    	poa->life_flags |= ORBit_LifeF_Deactivated;
-    poa->life_flags &= ~ORBit_LifeF_Deactivating;
-    return done;
+	if (poa->p_servant_retention == PortableServer_RETAIN) {
+		TraverseInfo	info;
+		info.num_in_use = 0;
+		info.do_deact = 1;
+		info.poa = poa;
+		info.do_etherealize = (poa->life_flags&ORBit_LifeF_DoEtherealize);
+		info.is_cleanup = TRUE;
+		g_assert (poa->oid_to_obj_map);
+		g_hash_table_foreach (
+			poa->oid_to_obj_map, (GHFunc)traverse_cb, &info);
+		g_hash_table_foreach_remove (
+			poa->oid_to_obj_map, (GHRFunc)remove_cb, NULL);
+		done = info.num_in_use == 0 ? CORBA_TRUE : CORBA_FALSE;
+	}
+
+	if (done)
+		poa->life_flags |= ORBit_LifeF_Deactivated;
+	poa->life_flags &= ~ORBit_LifeF_Deactivating;
+
+	return done;
 }
 
 void
@@ -508,7 +499,7 @@ ORBit_POA_free_fn (ORBit_RootObject obj)
 	ORBit_free_T (((ORBit_ObjectAdaptor)poa)->adaptor_key._buffer);
 
 	poa->orb = NULL;
-	g_free (poa);
+	p_free (poa, struct PortableServer_POA_type);
 }
 
 static const ORBit_RootObject_Interface ORBit_POA_epv = {
@@ -525,7 +516,7 @@ ORBit_ObjectId_sysid_hash (gconstpointer ptr)
 	return *(guint*) oid->_buffer;
 }
 
-guint
+static guint
 ORBit_sequence_CORBA_octet_hash (gconstpointer ptr)
 {
 	const CORBA_sequence_CORBA_octet *so = ptr;
@@ -547,7 +538,7 @@ ORBit_sequence_CORBA_octet_hash (gconstpointer ptr)
  * Returns TRUE if s1 and s2 are the same, FALSE otherwise.
  * Note that this is what the glib hash module wants.
  */
-gint
+static gint
 ORBit_sequence_CORBA_octet_equal (gconstpointer p1, gconstpointer p2)
 {
 	const CORBA_sequence_CORBA_octet *s1 = p1;
@@ -666,7 +657,7 @@ ORBit_POA_new (CORBA_ORB orb, const CORBA_char *nom,
   
 	poa = g_new0 (struct PortableServer_POA_type, 1);
 
-	ORBit_RootObject_init ((ORBit_RootObject)poa, &ORBit_POA_epv);
+	ORBit_RootObject_init ((ORBit_RootObject) poa, &ORBit_POA_epv);
 	/* released in ORBit_POA_destroy */
 	ORBit_RootObject_duplicate (poa);
 
@@ -743,10 +734,9 @@ ORBit_POA_obj_to_ref (PortableServer_POA  poa,
 		      CORBA_Environment  *ev)
 {
 	PortableServer_ObjectId *oid;
-	CORBA_Object             objref;
 	const char              *type_id = intf;
 
-	g_assert (pobj && ((ORBit_OAObject) pobj)->objref == NULL);
+	g_assert (pobj && !pobj->base.objref);
 
 	if (!type_id) {
 		g_assert (pobj->servant);
@@ -757,15 +747,13 @@ ORBit_POA_obj_to_ref (PortableServer_POA  poa,
 
 	oid = pobj->object_id;
 
-	objref = ORBit_objref_new (poa->poa_manager->orb,
-				   g_quark_from_string (type_id));
+	pobj->base.objref = ORBit_objref_new (poa->poa_manager->orb,
+					      g_quark_from_string (type_id));
 
 	/* released by CORBA_Object_release */
-	objref->adaptor_obj = ORBit_RootObject_duplicate (pobj);
+	pobj->base.objref->adaptor_obj = ORBit_RootObject_duplicate (pobj);
 
-	((ORBit_OAObject)pobj)->objref = ORBit_RootObject_duplicate (objref);
-
-	return objref;
+	return ORBit_RootObject_duplicate (pobj->base.objref);
 }
 
 PortableServer_POA
@@ -796,61 +784,58 @@ ORBit_POAObject_is_active (ORBit_POAObject pobj)
 	return FALSE;
 }
 
-/*
- * POAObject RootObject stuff
- */
+/* POAObject RootObject stuff */
 static void
-ORBit_POAObject_release_cb(ORBit_RootObject robj)
+ORBit_POAObject_release_cb (ORBit_RootObject robj)
 {
-  ORBit_POAObject    pobj = (ORBit_POAObject) robj;
-  PortableServer_POA poa = pobj->poa;
-  PortableServer_ObjectId *object_id;
+	ORBit_POAObject    pobj = (ORBit_POAObject) robj;
+	PortableServer_POA poa = pobj->poa;
+	PortableServer_ObjectId *object_id;
  
-  /* object *must* be deactivated */
-  g_assert (pobj->servant == NULL);
+	/* object *must* be deactivated */
+	g_assert (pobj->servant == NULL);
 
-  object_id = pobj->object_id;
-  pobj->object_id = NULL;
+	object_id = pobj->object_id;
+	pobj->object_id = NULL;
 
-  /*
-   * Don't want to remove from oid_to_obj_map if we 
-   * are currently traversing across it !
-   * Just mark it as destroyed
-   */
-  if ((poa->life_flags & ORBit_LifeF_Deactivating) == 0) {
-    g_hash_table_remove (poa->oid_to_obj_map, object_id);
-    g_free (pobj);
-  }
-  else
-    pobj->life_flags = ORBit_LifeF_Destroyed;
+	/*
+	 * Don't want to remove from oid_to_obj_map if we 
+	 * are currently traversing across it !
+	 * Just mark it as destroyed
+	 */
+	if ((poa->life_flags & ORBit_LifeF_Deactivating) == 0) {
+		g_hash_table_remove (poa->oid_to_obj_map, object_id);
+		p_free (robj, struct ORBit_POAObject_type);
+	} else
+		pobj->life_flags = ORBit_LifeF_Destroyed;
 
-  object_id->_release = CORBA_TRUE;
-  ORBit_free_T (object_id);
+	object_id->_release = CORBA_TRUE;
+	ORBit_free_T (object_id);
 
-  ORBit_RootObject_release_T (poa);
+	ORBit_RootObject_release_T (poa);
 }
 
 static ORBit_RootObject_Interface ORBit_POAObject_if = {
-  ORBIT_ROT_OAOBJECT,
-  ORBit_POAObject_release_cb
-  };
+	ORBIT_ROT_OAOBJECT,
+	ORBit_POAObject_release_cb
+};
 
 static struct 
 ORBit_OAObject_Interface_type ORBit_POAObject_methods = {
 	ORBIT_ADAPTOR_POA,
-	(ORBitStateCheckFunc)ORBit_POAObject_is_active,
-	(ORBitKeyGenFunc)ORBit_POAObject_object_to_objkey,
-	(ORBitInvokeFunc)ORBit_POAObject_invoke,
-	(ORBitReqFunc) ORBit_POAObject_handle_request
+	(ORBitStateCheckFunc) ORBit_POAObject_is_active,
+	(ORBitKeyGenFunc)     ORBit_POAObject_object_to_objkey,
+	(ORBitInvokeFunc)     ORBit_POAObject_invoke,
+	(ORBitReqFunc)        ORBit_POAObject_handle_request
 };
 
 /*
-    If USER_ID policy, {oid} must be non-NULL.
-    If SYSTEM_ID policy, {oid} must ether be NULL, or must have
-    been previously created by the POA. If the user passes in
-    a bogus oid under SYSTEM_ID, we will assert or segfault. This
-    is allowed by the CORBA spec.
-*/
+ *    If USER_ID policy, {oid} must be non-NULL.
+ *  If SYSTEM_ID policy, {oid} must ether be NULL, or must have
+ *  been previously created by the POA. If the user passes in
+ *  a bogus oid under SYSTEM_ID, we will assert or segfault. This
+ *  is allowed by the CORBA spec.
+ */
 static ORBit_POAObject
 ORBit_POA_create_object (PortableServer_POA             poa,
 			 const PortableServer_ObjectId *objid,
@@ -879,8 +864,7 @@ ORBit_POA_create_object (PortableServer_POA             poa,
 		}
 		else
 			newobj->object_id = ORBit_POA_new_system_objid (poa);
-	}
-	else {
+	} else {
 		newobj->object_id           = PortableServer_ObjectId__alloc ();
 		newobj->object_id->_length  = objid->_length;
 		newobj->object_id->_buffer  = PortableServer_ObjectId_allocbuf (objid->_length);
@@ -889,7 +873,7 @@ ORBit_POA_create_object (PortableServer_POA             poa,
 		memcpy(newobj->object_id->_buffer, objid->_buffer, objid->_length);
 	}
 
-	g_hash_table_insert(poa->oid_to_obj_map, newobj->object_id, newobj);
+	g_hash_table_insert (poa->oid_to_obj_map, newobj->object_id, newobj);
 
 	return newobj;
 }
@@ -932,73 +916,78 @@ ORBit_POA_activate_object(PortableServer_POA poa,
     ORBit_RootObject_duplicate(pobj);
 }
 
-
 /*
-    Note that this doesn't necessarily remove the object from
-    the oid_to_obj_map; it just removes knowledge of the servant.
-    If object is currentin use (servicing a request), etherialization
-    and memory release will occur later.
-**/
+ * Note that this doesn't necessarily remove the object from
+ * the oid_to_obj_map; it just removes knowledge of the servant.
+ * If the object is currently in use (servicing a request),
+ * etherialization and memory release will occur later.
+ */
 static void
-ORBit_POA_deactivate_object(PortableServer_POA poa, ORBit_POAObject pobj,
-			     CORBA_boolean do_etherealize,
-			     CORBA_boolean is_cleanup)
+ORBit_POA_deactivate_object (PortableServer_POA poa,
+			     ORBit_POAObject    pobj,
+			     CORBA_boolean      do_etherealize,
+			     CORBA_boolean      is_cleanup)
 {
-    PortableServer_ServantBase	*serv = pobj->servant;
-    GSList                      **obj_list;
+	GSList **obj_list;
+	PortableServer_ServantBase *serv = pobj->servant;
 
-    if ( serv == NULL ) {
-	    /* deactivation has already occured, or is in progress */
-	    return;
-    }
-    if ( do_etherealize && (pobj->life_flags&ORBit_LifeF_DeactivateDo)==0 )
-    	pobj->life_flags |= ORBit_LifeF_DoEtherealize;
-    if ( is_cleanup )
-    	pobj->life_flags |= ORBit_LifeF_IsCleanup;
-    if ( pobj->use_cnt > 0 ) {
-        pobj->life_flags |= ORBit_LifeF_DeactivateDo;
-	pobj->life_flags |= ORBit_LifeF_NeedPostInvoke;
-	return;
-    }
-    pobj->servant = NULL;
-    obj_list = ORBIT_SERVANT_TO_POAOBJECT_LIST_ADDR(serv);
-    *obj_list = g_slist_remove(*obj_list,pobj);
+	if (!serv) /* deactivation done, or in progress */
+		return;
 
-    if ( (pobj->life_flags & ORBit_LifeF_DoEtherealize)!=0 ) {
-	CORBA_Environment env, *ev = &env;
-	CORBA_exception_init(ev);
-	++(pobj->use_cnt);	/* prevent re-activation */
-        if ( poa->p_request_processing == PortableServer_USE_SERVANT_MANAGER) {
-	    POA_PortableServer_ServantActivator__epv *epv;
-	    POA_PortableServer_ServantActivator *sm;
+	if (do_etherealize && !(pobj->life_flags & ORBit_LifeF_DeactivateDo))
+		pobj->life_flags |= ORBit_LifeF_DoEtherealize;
+	
+	if (is_cleanup)
+		pobj->life_flags |= ORBit_LifeF_IsCleanup;
 
-	    sm = (POA_PortableServer_ServantActivator *)poa->servant_manager;
-	    epv = sm->vepv->PortableServer_ServantActivator_epv;
-	    (*(epv->etherealize))(sm, pobj->object_id, poa,
-			    serv,
-			    (pobj->life_flags & ORBit_LifeF_IsCleanup)
-			      ? CORBA_TRUE : CORBA_FALSE,
-			    /* remaining_activations */ CORBA_FALSE,
-			    ev);
-        }
-        {
-	    PortableServer_ServantBase__epv *epv = serv->vepv[0];
-	    /* In theory, the finalize fnc should always be non-NULL;
-	     * however, for backward compat. and general extended
-	     * applications we dont insist on it.
-	     */
-	    if ( epv && epv->finalize ) {
-	        (*(epv->finalize))(serv, ev);
-	    }
+	if (pobj->use_cnt > 0) {
+		pobj->life_flags |= ORBit_LifeF_DeactivateDo;
+		pobj->life_flags |= ORBit_LifeF_NeedPostInvoke;
+		return;
 	}
-	--(pobj->use_cnt);	/* allow re-activation */
-	g_assert( ev->_major == 0 );
-    }
+	pobj->servant = NULL;
 
-    pobj->life_flags &= ~(ORBit_LifeF_DeactivateDo
-      |ORBit_LifeF_IsCleanup|ORBit_LifeF_DoEtherealize);
+	obj_list = ORBIT_SERVANT_TO_POAOBJECT_LIST_ADDR (serv);
+	*obj_list = g_slist_remove (*obj_list, pobj);
 
-    ORBit_RootObject_release(pobj);
+	if (pobj->life_flags & ORBit_LifeF_DoEtherealize) {
+		CORBA_Environment env, *ev = &env;
+
+		CORBA_exception_init (ev);
+
+		pobj->use_cnt++; /* prevent re-activation */
+		if (poa->p_request_processing == PortableServer_USE_SERVANT_MANAGER) {
+			POA_PortableServer_ServantActivator      *sm;
+			POA_PortableServer_ServantActivator__epv *epv;
+
+			sm = (POA_PortableServer_ServantActivator *) poa->servant_manager;
+			epv = sm->vepv->PortableServer_ServantActivator_epv;
+
+			epv->etherealize (sm, pobj->object_id, poa,
+					  serv,
+					  pobj->life_flags & ORBit_LifeF_IsCleanup,
+					  /* remaining_activations */ CORBA_FALSE,
+					  ev);
+		}
+		{
+			PortableServer_ServantBase__epv *epv = serv->vepv[0];
+			/* In theory, the finalize fnc should always be non-NULL;
+			 * however, for backward compat. and general extended
+			 * applications we dont insist on it. */
+			if (epv && epv->finalize)
+				epv->finalize (serv, ev);
+		}
+		pobj->use_cnt--; /* allow re-activation */
+		g_assert (ev->_major == 0);
+
+		CORBA_exception_free (ev);
+	}
+
+	pobj->life_flags &= ~(ORBit_LifeF_DeactivateDo |
+			      ORBit_LifeF_IsCleanup |
+			      ORBit_LifeF_DoEtherealize);
+
+	ORBit_RootObject_release (pobj);
 }
 
 struct ORBit_POA_invoke_data {
@@ -1080,9 +1069,13 @@ ORBit_POAObject_handle_request (ORBit_POAObject    pobj,
 		switch (poa->poa_manager->state) {
 
 		case PortableServer_POAManager_HOLDING:
-			if (recv_buffer)
+			if (recv_buffer) {
+				g_warning ("POAManager in holding state. "
+					   "Queueing '%s' method request", opname);
+						
 				poa->held_requests = g_slist_prepend (
 					poa->held_requests, recv_buffer);
+			}
 			else
 				CORBA_exception_set_system (
 					ev, ex_CORBA_TRANSIENT,
@@ -1165,14 +1158,19 @@ ORBit_POAObject_handle_request (ORBit_POAObject    pobj,
 			(gpointer *)&m_data, &imp);
 	
 	if ((!small_skel && !skel) || !imp) {
-		if (!imp && (small_skel || skel))
+		if (!imp && (small_skel || skel)) {
+			tprintf ("'%s' not implemented on %p",
+				 opname, pobj);
 			CORBA_exception_set_system (
 				ev, ex_CORBA_NO_IMPLEMENT,
 				CORBA_COMPLETED_NO);
-		else
+		} else {
+			tprintf ("Bad operation '%s' on %p",
+				 opname, pobj);
 			CORBA_exception_set_system (
 				ev, ex_CORBA_BAD_OPERATION,
 				CORBA_COMPLETED_NO);
+		}
 	}
 
 	if (ev->_major != CORBA_NO_EXCEPTION) {
@@ -1196,7 +1194,8 @@ ORBit_POAObject_handle_request (ORBit_POAObject    pobj,
 		small_skel (pobj->servant, ret, args, ctx, ev, imp);
 
  clean_out:
-	CORBA_exception_free (ev);
+	if (recv_buffer)
+		CORBA_exception_free (ev);
 
 	if (poa->p_servant_retention == PortableServer_NON_RETAIN)
 		switch (poa->p_request_processing) {
@@ -1234,6 +1233,9 @@ ORBit_POA_handle_request (PortableServer_POA poa,
 
 	CORBA_exception_init (&env);
 
+	/* FIXME: we can return unexpected excptions from oneway
+	   methods like this ! */
+
 	if (!ORBit_POA_okey_to_objid (poa, objkey, &oid)) {
 		CORBA_exception_set_system (&env, 
 					    ex_CORBA_OBJECT_NOT_EXIST,
@@ -1278,9 +1280,9 @@ ORBit_POA_handle_request (PortableServer_POA poa,
 		GIOPSendBuffer *reply_buf;
 
 		reply_buf = giop_send_buffer_use_reply ( 
-					recv_buffer->connection->giop_version,
-					giop_recv_buffer_get_request_id (recv_buffer),
-					CORBA_SYSTEM_EXCEPTION);
+			recv_buffer->connection->giop_version,
+			giop_recv_buffer_get_request_id (recv_buffer),
+			CORBA_SYSTEM_EXCEPTION);
 		ORBit_send_system_exception (reply_buf, &env);
 		giop_send_buffer_write (reply_buf, recv_buffer->connection);
 		giop_send_buffer_unuse (reply_buf);
@@ -1336,7 +1338,7 @@ ORBit_POA_ServantManager_use_servant( PortableServer_POA poa,
 	  *obj_list = g_slist_append(*obj_list,pobj);
 
           /* released by ORBit_POA_deactivate_object */
-	  ORBit_RootObject_duplicate(pobj);
+	  ORBit_RootObject_duplicate (pobj);
 	  pobj->servant = retval;
 	}
     }
@@ -1530,10 +1532,7 @@ PortableServer_Current_get_POA (PortableServer_Current  obj,
 
 	pobj = ORBit_POACurrent_get_object (obj, ev);
 
-	if (!pobj) 
-		return CORBA_OBJECT_NIL;
-
-	return (PortableServer_POA)CORBA_Object_duplicate ((CORBA_Object)pobj->poa, ev);
+	return ORBit_RootObject_duplicate (pobj->poa);
 }
 
 PortableServer_ObjectId *
@@ -1597,7 +1596,7 @@ PortableServer_POA_find_POA (PortableServer_POA   poa,
 			   adapter_name);
 
 	if (child_poa)
-		CORBA_Object_duplicate ((CORBA_Object)child_poa, ev);
+		ORBit_RootObject_duplicate (child_poa);
 	else
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_PortableServer_POA_AdapterNonExistent,
@@ -1639,8 +1638,7 @@ PortableServer_POA
 PortableServer_POA__get_the_parent (PortableServer_POA  poa,
 				    CORBA_Environment  *ev)
 {
-	return (PortableServer_POA)
-			CORBA_Object_duplicate ((CORBA_Object)poa->parent_poa, ev);
+	return ORBit_RootObject_duplicate (poa->parent_poa);
 }
 
 PortableServer_POAList*
@@ -1664,16 +1662,14 @@ PortableServer_POAManager
 PortableServer_POA__get_the_POAManager (PortableServer_POA  poa,
 					CORBA_Environment  *ev)
 {
-	return (PortableServer_POAManager)
-			CORBA_Object_duplicate ((CORBA_Object)poa->poa_manager, ev);
+	return ORBit_RootObject_duplicate (poa->poa_manager);
 }
 
 PortableServer_AdapterActivator
 PortableServer_POA__get_the_activator (PortableServer_POA  poa,
 				       CORBA_Environment  *ev)
 {
-	return (PortableServer_AdapterActivator)
-			CORBA_Object_duplicate ((CORBA_Object)poa->the_activator, ev);
+	return ORBit_RootObject_duplicate (poa->the_activator);
 }
 
 void
@@ -1692,8 +1688,7 @@ PortableServer_ServantManager
 PortableServer_POA_get_servant_manager (PortableServer_POA  poa,
 					CORBA_Environment  *ev)
 {
-	return (PortableServer_ServantManager)
-			CORBA_Object_duplicate ((CORBA_Object)poa->servant_manager, ev);
+	return ORBit_RootObject_duplicate (poa->servant_manager);
 }
 
 void
@@ -1862,21 +1857,15 @@ PortableServer_POA_servant_to_id (PortableServer_POA            poa,
 {
 	PortableServer_ServantBase *servant = p_servant;
 	ORBit_POAObject             pobj = ORBIT_SERVANT_TO_FIRST_POAOBJECT (servant);
-	gboolean                    defserv, retain, implicit, unique;
+	gboolean                    defserv = IS_USE_DEFAULT_SERVANT (poa);
+	gboolean                    retain = IS_RETAIN (poa);
+	gboolean                    implicit = IS_IMPLICIT_ACTIVATION (poa);
+	gboolean                    unique = IS_UNIQUE_ID (poa);
 
-	g_return_val_if_fail (p_servant, NULL);
+	poa_sys_exception_val_if_fail (servant, ex_CORBA_BAD_PARAM, NULL);
 
-	defserv   = (poa->p_request_processing  == PortableServer_USE_DEFAULT_SERVANT);
-	retain    = (poa->p_servant_retention   == PortableServer_RETAIN);
-	implicit  = (poa->p_implicit_activation == PortableServer_IMPLICIT_ACTIVATION);
-	unique    = (poa->p_id_uniqueness       == PortableServer_UNIQUE_ID);
-
-	if (!(defserv || (retain && (unique || implicit)))) {
-		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-				     ex_PortableServer_POA_WrongPolicy,
-				     NULL);
-		return NULL;
-	}
+	poa_exception_val_if_fail (defserv || (retain && (unique || implicit)),
+				   ex_PortableServer_POA_WrongPolicy, NULL);
 
 	if (retain && unique && pobj && pobj->servant == p_servant)
 		return ORBit_sequence_CORBA_octet_dup (pobj->object_id);
@@ -1920,22 +1909,20 @@ PortableServer_POA_servant_to_reference (PortableServer_POA            poa,
 {
 	PortableServer_ServantBase *servant = p_servant;
 	ORBit_POAObject             pobj = ORBIT_SERVANT_TO_FIRST_POAOBJECT (servant);
-	gboolean                    retain, implicit, unique;
+	gboolean                    retain = IS_RETAIN (poa);
+	gboolean                    implicit = IS_IMPLICIT_ACTIVATION (poa);
+	gboolean                    unique = IS_UNIQUE_ID (poa);
 
-	retain   = (poa->p_servant_retention   == PortableServer_RETAIN);
-	implicit = (poa->p_implicit_activation == PortableServer_IMPLICIT_ACTIVATION);
-	unique   = (poa->p_id_uniqueness       == PortableServer_UNIQUE_ID);
+	poa_sys_exception_val_if_fail (servant, ex_CORBA_BAD_PARAM, CORBA_OBJECT_NIL);
 
-	if (!(retain && (unique || implicit))) {
-		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-				     ex_PortableServer_POA_WrongPolicy,
-				     NULL);
-		return CORBA_OBJECT_NIL;
-	}
+	poa_exception_val_if_fail (retain && (unique || implicit),
+				   ex_PortableServer_POA_WrongPolicy,
+				   CORBA_OBJECT_NIL);
 
 	if (retain && unique && pobj)
-		if (((ORBit_OAObject)pobj)->objref)
-			return ((ORBit_OAObject)pobj)->objref;
+		if (pobj->base.objref)
+			return ORBit_RootObject_duplicate (
+				pobj->base.objref);
 		else
 			return ORBit_POA_obj_to_ref (poa, pobj, NULL, ev);
 
@@ -1974,39 +1961,28 @@ PortableServer_POA_reference_to_servant (PortableServer_POA  poa,
 					 const CORBA_Object  reference,
 					 CORBA_Environment  *ev)
 {
-	if (reference == CORBA_OBJECT_NIL)
-		return NULL;
+	poa_sys_exception_val_if_fail (reference != CORBA_OBJECT_NIL,
+				       ex_CORBA_BAD_PARAM, NULL);
 
-	if (poa->p_request_processing != PortableServer_USE_DEFAULT_SERVANT && 
-	    poa->p_servant_retention  != PortableServer_RETAIN) {
-		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-				     ex_PortableServer_POA_WrongPolicy,
-				     NULL);
-		return NULL;
-	}
+	poa_exception_val_if_fail (IS_USE_DEFAULT_SERVANT (poa) || IS_RETAIN (poa),
+				   ex_PortableServer_POA_WrongPolicy, NULL);
 
 	/*
-	 * FIXME: Is this correct
+	 * FIXME: Is this correct?
 	 */
-	if (poa->p_servant_retention == PortableServer_RETAIN ) {
-		if (!reference->adaptor_obj) {
-			CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-					     ex_PortableServer_POA_WrongAdapter,
-					     NULL);
-			return NULL;
-		}
+	if (IS_RETAIN (poa)) {
+		poa_exception_val_if_fail (reference->adaptor_obj,
+					   ex_PortableServer_POA_WrongAdapter, 
+					   NULL);
 
 		if (((ORBit_POAObject)reference->adaptor_obj)->servant)
 			return ((ORBit_POAObject)reference->adaptor_obj)->servant;
 	}
 
-	if (poa->p_request_processing == PortableServer_USE_DEFAULT_SERVANT &&
-	    poa->default_servant)
+	if (IS_USE_DEFAULT_SERVANT (poa) && poa->default_servant)
 		return poa->default_servant;
 
-	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-			     ex_PortableServer_POA_ObjectNotActive,
-			     NULL);
+	poa_exception_val_if_fail (FALSE, ex_PortableServer_POA_ObjectNotActive, NULL);
 
 	return NULL;
 }
@@ -2018,17 +1994,15 @@ PortableServer_POA_reference_to_id (PortableServer_POA  poa,
 {
 	ORBit_POAObject pobj;
 
-	if (reference == CORBA_OBJECT_NIL)
-		return NULL;
+	poa_sys_exception_val_if_fail (reference != CORBA_OBJECT_NIL,
+				       ex_CORBA_BAD_PARAM, NULL);
 
 	pobj = (ORBit_POAObject)reference->adaptor_obj;
 	if (pobj)
 		return (PortableServer_ObjectId *)
 				ORBit_sequence_CORBA_octet_dup (pobj->object_id);
 
-	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-			     ex_PortableServer_POA_WrongAdapter,
-			     NULL);
+	poa_exception_val_if_fail (FALSE, ex_PortableServer_POA_WrongAdapter, NULL);
 
 	return NULL;
 }
@@ -2038,29 +2012,22 @@ PortableServer_POA_id_to_servant (PortableServer_POA             poa,
 				  const PortableServer_ObjectId *oid,
 				  CORBA_Environment             *ev)
 {
-	if (poa->p_request_processing != PortableServer_USE_DEFAULT_SERVANT &&
-	    poa->p_servant_retention  != PortableServer_RETAIN) {
+	poa_sys_exception_val_if_fail (oid, ex_CORBA_BAD_PARAM, NULL);
 
-		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-				     ex_PortableServer_POA_WrongPolicy,
-				     NULL);
-		return NULL;
-	}
+	poa_exception_val_if_fail (IS_USE_DEFAULT_SERVANT (poa) || IS_RETAIN (poa),
+				   ex_PortableServer_POA_WrongPolicy, NULL);
 
-	if (poa->p_servant_retention == PortableServer_RETAIN) {
+	if (IS_RETAIN (poa)) {
 		ORBit_POAObject pobj = ORBit_POA_oid_to_obj (poa, oid, TRUE, NULL);
 
 		if (pobj && pobj->servant)
 			return pobj->servant;
 	}
 
-	if (poa->p_request_processing == PortableServer_USE_DEFAULT_SERVANT &&
-	    poa->default_servant)
+	if (IS_USE_DEFAULT_SERVANT (poa) && poa->default_servant)
 		return poa->default_servant;
 
-	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-			     ex_PortableServer_POA_ObjectNotActive,
-			     NULL);
+	poa_exception_val_if_fail (FALSE, ex_PortableServer_POA_ObjectNotActive, NULL);
 
 	return NULL;
 }
@@ -2072,12 +2039,14 @@ PortableServer_POA_id_to_reference (PortableServer_POA             poa,
 {
 	ORBit_POAObject pobj;
 
+	poa_sys_exception_val_if_fail (oid, ex_CORBA_BAD_PARAM, CORBA_OBJECT_NIL);
+
 	pobj = ORBit_POA_oid_to_obj (poa, oid, TRUE, ev);
 	if (ev->_major != CORBA_NO_EXCEPTION)
 		return NULL;
 
-	if (((ORBit_OAObject)pobj)->objref != CORBA_OBJECT_NIL)
-		return ((ORBit_OAObject)pobj)->objref;
+	if (pobj->base.objref)
+		return ORBit_RootObject_duplicate (pobj->base.objref);
 
 	return ORBit_POA_obj_to_ref (poa, pobj, NULL, ev);
 }
